﻿using IceCreamShopContracts.BindingModels;
using IceCreamShopContracts.SearchModels;
using IceCreamShopContracts.StoragesContracts;
using IceCreamShopContracts.ViewModels;
using IceCreamShopDatabaseImplement.Models;
using Microsoft.EntityFrameworkCore;

namespace IceCreamShopDatabaseImplement.Implements
{
    public class OrderStorage : IOrderStorage
    {
        public List<OrderViewModel> GetFullList()
        {
            using var context = new IceCreamShopDatabase();
            return context.Orders
                    .Include(x => x.IceCream)
                    .Include(x => x.Client)
                    .Select(x => x.GetViewModel)
                    .ToList();
        }

        public List<OrderViewModel> GetFilteredList(OrderSearchModel model)
        {
            using var context = new IceCreamShopDatabase();
            if (model.Id.HasValue)
            {
                return context.Orders
                    .Include(x => x.IceCream)
                    .Include(x => x.Client)
                    .Where(x => x.Id == model.Id)
                    .Select(x => x.GetViewModel)
                    .ToList();
            }
            else if (model.DateFrom != null && model.DateTo != null)
            {
                return context.Orders
                    .Include(x => x.IceCream)
                    .Include(x => x.Client)
                    .Where(x => x.DateCreate >= model.DateFrom && x.DateCreate <= model.DateTo)
                    .Select(x => x.GetViewModel)
                    .ToList();
            }
            else if (model.ClientId.HasValue)
            {
                return context.Orders
                    .Include(x => x.IceCream)
                    .Include(x => x.Client)
                    .Where(x => x.ClientId == model.ClientId)
                    .Select(x => x.GetViewModel)
                    .ToList();
            }
            return new();
        }

        public OrderViewModel? GetElement(OrderSearchModel model)
        {
            if (!model.Id.HasValue)
            {
                return null;
            }
            using var context = new IceCreamShopDatabase();
            return context.Orders
                    .Include(x => x.IceCream)
                    .Include(x => x.Client)
                    .FirstOrDefault(x => x.Id == model.Id)              
                    ?.GetViewModel;
        }

        public OrderViewModel? Insert(OrderBindingModel model)
        {
            var newOrder = Order.Create(model);
            if (newOrder == null)
            {
                return null;
            }
            using var context = new IceCreamShopDatabase();
            context.Orders.Add(newOrder);
            context.SaveChanges();
            return context.Orders
                      .Include(x => x.IceCream)
                      .Include(x => x.Client)
                      .FirstOrDefault(x => x.Id == newOrder.Id)
                      ?.GetViewModel;
        }

        public OrderViewModel? Update(OrderBindingModel model)
        {
            using var context = new IceCreamShopDatabase();
            var order = context.Orders.FirstOrDefault(x => x.Id == model.Id);
            if (order == null)
            {
                return null;
            }
            order.Update(model);
            context.SaveChanges();
            return context.Orders
                         .Include(x => x.IceCream)
                         .Include(x => x.Client)
                         .FirstOrDefault(x => x.Id == model.Id)
                         ?.GetViewModel;
        }

        public OrderViewModel? Delete(OrderBindingModel model)
        {
            using var context = new IceCreamShopDatabase();
            var element = context.Orders.FirstOrDefault(rec => rec.Id == model.Id);
            if (element != null)
            {
                var deletedElement = context.Orders
                                           .Include(x => x.IceCream)
                                           .Include(x => x.Client)
                                           .FirstOrDefault(x => x.Id == model.Id)
                                           ?.GetViewModel;
                context.Orders.Remove(element);
                context.SaveChanges();
                return deletedElement;
            }
            return null;
        }   
    }
}